#!/usr/bin/python
# coding=utf8
from sys import exc_info
"""
Comnt - template content using block-annotated comments

A dead-simple, human-friendly way to manage template regions using
**standard comment syntax** — no weird symbols, no broken files, no preprocessing
to view your code in a browser.

One huge practical benefit: you can inject actual JavaScript-ready values from Python—
not just strings or you can refer to javascript functions defined elsewhere.
That means you don’t need to wrap your data in quotes and then call
`JSON.parse(...)` on the frontend, like you often do with Jinja2.

Supported tag/comment Formats:
    HTML/XML style:
        <!--[tag_name-->
        Content to be replaced
        <!--tag_name]-->

    JavaScript/CSS style:
        /*[tag_name*/
       const arr_to_change = [0,1]
        /*tag_name]*/

Tag Format Rules:
    - Opening tag: Comment start + '[' + tag_name + Comment continuation
    - Closing tag: Comment start + tag_name + ']' + Comment end
    - Tag names must match exactly between opening and closing tags

Author: [Tomasz Sługocki]
Version: 0.0.1
"""

__all__ = ['render', 'write_from_template', 'simple_example', 'example']


class NotFoundError(Exception):
    pass


def _render_block(text, tag_id, val=None):
    assert isinstance(text, str) and isinstance(tag_id, str)
    assert len(text) > len(tag_id)

    assert val is None or isinstance(val, str)

    def get_tags(text, tag_id, ext):
        assert ext in ("html", "js")
        start, end = ("/*", "*/") if ext == "js" else ("<!--", "-->")
        start_tag_js, start_tag_html = f"/*[{tag_id}*/", f"<!--[{tag_id}-->"
        if start_tag_js in text and start_tag_html in text:
            raise AssertionError(f"same tag id in javascript and html is not allowed")
        start_tag = "".join((start, "[", tag_id, end))
        end_tag = "".join((start, tag_id, "]", end))
        start_cnt = len(text.split(start_tag)) - 1
        end_cnt = len(text.split(end_tag)) - 1
        if not start_cnt and not end_cnt:
            raise ValueError(f"not found tags {start_tag} {end_tag}, ext:{ext}")
        if not start_cnt:
            raise NotFoundError(f"start tag not found in form: '{start_tag}'")
        if not end_cnt:
            raise NotFoundError(f"end tag not found in form: '{end_tag}'")
        if start_cnt > 1:
            raise AssertionError(f"more than one start tag: '{start_tag}'")
        if end_cnt > 1:
            raise AssertionError(f"more than one end tag: '{end_tag}'")
        return start_tag, end_tag

    start_tag, end_tag = None, None
    errs = []
    for ext in ["js", "html"]:
        try:
            start_tag, end_tag = get_tags(text, tag_id, ext)
        except ValueError:
            errs.append(repr(exc_info()[1]))
    if not start_tag:
        raise ValueError(f"Errors happened: {''.join(errs)}")

    start_idx = text.find(start_tag)
    end_idx = text.find(end_tag)
    prefix = text[:start_idx + len(start_tag)]
    suffix = text[end_idx:]

    if val is None:  # empty strings are ok
        old_val = text[start_idx + len(start_tag):end_idx]
        return old_val
    return "\n".join((prefix, val, suffix))


def get_tag_content(tag, instr):
    assert len(instr) > len(tag) and tag + instr
    return _render_block(instr, tag, None)


def render(instr, repldict):
    for key, val in repldict.items():
        try:
            instr = _render_block(instr, key, val)
        except ValueError:
            print(exc_info()[1])

    return instr


def write_from_template(template, newfile, repldict):
    assert isinstance(repldict, dict)
    assert template != newfile
    with open(template, encoding="utf-8") as op_file:
        instr = op_file.read()
    replaced = render(instr, repldict)
    with open(newfile, "w", encoding="utf8") as outfile:
        outfile.write(replaced)
    return True


def example():
    import os
    import subprocess
    import sys

    def open_file(filename):
        if sys.platform.startswith("win"):
            os.startfile(filename)
        else:
            opener = "open" if sys.platform == "darwin" else "xdg-open"
            subprocess.call([opener, filename])

    content = """
    <!DOCTYPE html>
    <html>
       <head>
          <title>
             Example  comnt rendered page
            </title>

       </head>
       <body> <h3>
           <!--[title-->
             Welcome to our site!

          <!--title]--></h3>
          <!--[content-->

          <p>
             This is placeholder content that shows in the browser.
          </p>
          <!--content]-->
          <code id="data-display" style=
             "background-color: rgb(245, 245, 245); padding: 10px; border: 1px solid rgb(221, 221, 221);">
             </code>
          <script>
             // Example data array for
             const data = /*[data_arr*/ [0, 1]
             /*data_arr]*/;

             document.addEventListener('DOMContentLoaded', function() {
                 // Display data array in paragraph (like Python's repr)
                 const dataDisplay = document.getElementById('data-display');
                 dataDisplay.textContent = JSON.stringify(data, null, 2);
             });
          </script>
       </body>
    </html>
    """

    outstr = render(
        content,
        {
            "title": "Example rendered python object",
            "content": "<p>Below python range rendered as javascript variable</p>",
            "data_arr": repr(list(range(10))),  # dont even need json here
            # , 'none_existing' : '0'
        },
    )
    file_name = os.path.join(os.getcwd(), "comnt_test.html")
    with open(file_name, "w", encoding="utf8") as outfile:
        outfile.write(outstr)
    open_file(file_name)


def simple_example():
    import json
    import os
    import subprocess
    import sys

    def open_file(filename):
        if sys.platform.startswith("win"):
            os.startfile(filename)
        else:
            opener = "open" if sys.platform == "darwin" else "xdg-open"
            subprocess.call([opener, filename])

    content = """
    <!DOCTYPE html>
    <p id="title"> </p>
    <div id="data_arr"></div>
    <script>
    const title = /*[title*/
        "Example title"
    /*title]*/;
    const data = /*[data_arr*/
        [0, 1]
    /*data_arr]*/;

      document.getElementById("data_arr").textContent = JSON.stringify(data);
      document.getElementById("title").textContent = title;
    </script>
    """
    outstr = render(
        content,
        {
            "title": json.dumps("Example rendered python object"),
            "data_arr": json.dumps(list(range(10))),
        },
    )
    file_name = os.path.join(os.getcwd(), "comnt_simple.html")
    with open(file_name, "w", encoding="utf8") as outfile:
        outfile.write(outstr)
    open_file(file_name)


if __name__ == "__main__":
    # simple_example()
    example()



import json
import math
import os
import subprocess
import sys
from functools import partial

import numpy as np
import pandas as pd

TEMPLATE_FILE = "par_template.html"
try:
    # python 3.9+
    from importlib import resources

    TEMPLATE_PATH = str(resources.files("interpareto").joinpath(TEMPLATE_FILE))
except ImportError:
    TEMPLATE_PATH = os.path.join(os.path.dirname(os.path.abspath(__file__)), TEMPLATE_FILE)

try:
    from . import comnt
except ImportError:
    import comnt


def is_index_like(series, tolerance=0.1):
    # Check if series is empty
    if len(series) == 0:
        return False

    # Check if series has only one element
    if len(series) == 1:
        return True

    # Check if all values are numeric
    if not pd.api.types.is_numeric_dtype(series):
        return False

    # Check for NaN values
    if series.isnull().any():
        return False

    # Calculate differences between consecutive elements
    consecutive_diffs = series.diff().dropna()

    # Check if all differences are approximately equal (consistent step)
    if len(consecutive_diffs) > 0:
        first_step = consecutive_diffs.iloc[0]

        # For integer-like steps, use exact comparison
        if abs(first_step - round(first_step)) < tolerance:
            rounded_step = round(first_step)
            step_consistency = all(
                abs(diff - rounded_step) < tolerance for diff in consecutive_diffs)
        else:
            # For floating point steps, use tolerance-based comparison
            step_consistency = all(abs(diff - first_step) < tolerance for diff in consecutive_diffs)

        return step_consistency

    return False


def process_df(df: pd.DataFrame):
    assert len(df.index) > 5 and len(df.columns) > 0
    processed_df = df.copy()

    # Check if index is already string type
    if not (processed_df.index.dtype == "object" or
            pd.api.types.is_string_dtype(processed_df.index)):
        # does not help for plotly treats numeric strings as numbers
        processed_df.set_index(processed_df.index.map(lambda x: str(x)), inplace=True)

    processing_messages = []
    for column_name in processed_df.columns:
        column_values = processed_df[column_name]
        # print(column_name, is_index_like(column_values))
        if is_index_like(column_values):
            processing_messages.append(f"Dropped column '<b>{column_name}</b>': looks like index")
            processed_df.drop(columns=[column_name], axis=1, inplace=True)

    if not processed_df.index.is_unique:
        duplicate_counts = repr(processed_df.index.value_counts().nlargest(3))
        processing_messages.append(f"Dropped duplicate index: {duplicate_counts}")

        processed_df = processed_df[~processed_df.index.duplicated(keep="first")]

    # Select numeric columns and slice first 10
    numeric_columns = processed_df.select_dtypes(include="number")
    # print(numeric_columns)
    first_ten_numeric = numeric_columns.iloc[:, :10]

    for column_name in first_ten_numeric.columns:
        column_values = first_ten_numeric[column_name]
        # print(column_name, is_index_like(column_values))
        if is_index_like(column_values):
            print("!drop", column_name)
            processing_messages.append(f"Column '{column_name}': looks like index")
            first_ten_numeric.drop(columns=[column_name], axis=1, inplace=True)

        nan_count = column_values.isna().sum()
        negative_count = (column_values < 0).sum()
        zero_count = (column_values == 0).sum()

        if nan_count > 0 or negative_count > 0 or zero_count > 0:
            processing_messages.append(
                f"Column '<b>{column_name}</b>': {nan_count} NaNs, ({negative_count} negative values, {zero_count} zeros)"
            )
    #todo drop zeros?
    # first_ten_numeric = first_ten_numeric[~(first_ten_numeric == 0).any(axis=1)]

    messages_summary = ("</br>".join(processing_messages) if processing_messages else "")
    return first_ten_numeric, messages_summary


def open_file(filename):
    if sys.platform.startswith("win"):
        os.startfile(filename)
    else:
        file_opener = "open" if sys.platform == "darwin" else "xdg-open"
        subprocess.call([file_opener, filename])


def render(
    df,
    to_file=None,
    title="Pareto dashboard",
    templ_path=TEMPLATE_PATH,
    startfile=True,
    warnings=True,
):
    processed_df, processing_info = process_df(df)
    column_headers = ["index"] + processed_df.columns.tolist()

    def to_none(val):
        try:
            if math.isnan(val):
                return None
        except TypeError:
            return val
        return val

    tabular_data = []
    #safe way to deal with Nan's in json'
    for row_values in processed_df.to_records():
        tabular_data.append(list(map(to_none, row_values.tolist())))

    data_json_string = json.dumps(tabular_data, separators=(",", ":"))
    headers_json_string = json.dumps(column_headers, separators=(",", ":"))
    warning_info = f"<p class='warn'>{processing_info}</p>" if warnings and processing_info.strip() else ""

    template_variables = {
        "p_data": data_json_string,
        "col_names": headers_json_string,
        "title": title,
        "warn": warning_info,
    }
    with open(templ_path, encoding="utf-8") as template_file:
        template_content = template_file.read()

    rendered_html = comnt.render(template_content, template_variables)

    if not to_file:
        return rendered_html

    assert templ_path != to_file and templ_path not in to_file
    with open(to_file, "w", encoding="utf8") as output_file:
        output_file.write(rendered_html)

    if startfile:
        open_file(to_file)

    return output_file


_render_str = partial(render, to_file=None)


def render_inline(df, **kwargs):
    if "to_file" in kwargs:
        print(f"wrong argument:[to_file] {kwargs.pop('to_file')} is not allowed in render_inline")
        # del kwargs['to_file']

    inline_html = _render_str(df, **kwargs)
    minimal_content = comnt.render(comnt.get_tag_content("min_content", inline_html),
                                   {"render_inline": "true"})
    return minimal_content


def generate_pareto_data(N=15):

    def generate_smoothed_pareto_column(N, base_high=5000, noise=1):
        position_ranks = np.arange(1, N + 1)
        pareto_values = base_high / position_ranks  # decay with rank
        pareto_values += np.random.uniform(-noise, noise,
                                           size=N) * pareto_values  # smooth randomness
        np.random.shuffle(pareto_values)
        return pareto_values

    np.random.seed(42)
    sample_data = {
        "Range to drop": range(N),
        "smoothed_pareto_A": generate_smoothed_pareto_column(N, base_high=200),
        "smoothed_pareto_B": generate_smoothed_pareto_column(N, base_high=10000),
        "smoothed_pareto_C": generate_smoothed_pareto_column(N, base_high=6000),
        "normal_mean_30_std_5": np.random.normal(loc=30, scale=1, size=N).clip(min=0),
        "uniform_0_to_1000": np.random.uniform(-5, 10, size=N),
        "poisson_lambda_15": np.random.poisson(lam=15, size=N),
        "beta_scaled_0_to_100": (np.random.beta(a=2, b=5, size=N) * 100),
        "binomial_n100_p0.2": np.random.binomial(n=100, p=0.2, size=N),
        "zipf_a1.5_clipped": np.clip(np.random.zipf(a=1.5, size=N), 1, 500),
        "gamma_shape_2_scale_20": np.random.gamma(shape=2.0, scale=20.0, size=N),
    }

    result_df = pd.DataFrame(sample_data).round(2)
    return result_df


def main():
    output_filename = "demo.html"
    demo_df = generate_pareto_data(100)
    render(demo_df, to_file=output_filename, title='Pareto dashboard', warnings=False)


if __name__ == "__main__":
    main()
